use crate::web_fetchers;
use feed_rs::model::Entry;
use rayon::prelude::*;
use scraper::{Html, Selector};
use std::cmp::Reverse;
use std::fs;

pub fn read_feed(path: &str) -> Vec<Entry> {
    let binding = fs::read_to_string(path).unwrap();
    let feed_urls: Vec<&str> = binding.lines().collect();

    log::trace!("Fetching feeds:");
    let raw_entries: Vec<Result<Vec<Entry>, String>> = feed_urls
        .into_par_iter()
        .map(|url| {
            web_fetchers::fetch_feed(url)
                .map_err(|e| format!("Failed to fetch or parse feed {}: {}", url, e))
        })
        .collect();

    let mut entries: Vec<Entry> = Vec::new();
    for entry in raw_entries {
        match entry {
            Ok(mut feed_entries) => entries.append(&mut feed_entries),
            Err(e) => log::warn!("{}", e),
        }
    }

    // Remove any entries that don't have a timestamp, and then sort by timestamps
    entries.retain(|entry| entry.published.is_some() || entry.updated.is_some());
    entries.sort_by_key(|entry| Reverse(get_entry_date(entry)));

    entries
}

pub fn get_entry_date(entry: &Entry) -> chrono::DateTime<chrono::Utc> {
    entry.published.unwrap_or(entry.updated.unwrap_or_default())
}

pub fn get_root_url(input_url: &str) -> &str {
    let mut url = input_url;

    url = url.strip_prefix("https://").unwrap_or(url);
    url = url.strip_prefix("http://").unwrap_or(url);

    url.split_once('/').unwrap().0
}

pub fn truncate_description(description: &str, max_length: usize) -> String {
    let description_trimmed = description.trim();
    if description_trimmed.len() > max_length {
        let mut char_boundary = max_length;
        for (idx, _) in description_trimmed.char_indices() {
            if idx > max_length {
                break;
            }
            char_boundary = idx;
        }
        format!("{}...", &description_trimmed[..char_boundary])
    } else {
        description_trimmed.to_string()
    }
}

pub fn strip_html_tags(html: &str) -> String {
    let document = Html::parse_document(html);
    // Use the wildcard selector to select all nodes and extract their text.
    let selector = Selector::parse("*").unwrap();
    let mut text_content = String::new();

    for element in document.select(&selector) {
        let text = element.text().collect::<Vec<_>>().join(" ");
        text_content.push_str(&text);
        text_content.push(' ');
    }

    text_content.trim().to_string()
}

pub fn group_by_nth<T: Clone>(slice: &[T], n: usize) -> Vec<Vec<T>> {
    (0..n)
        .map(|i| {
            slice
                .iter()
                .enumerate()
                .filter_map(|(index, value)| {
                    if index % n == i {
                        Some(value.clone())
                    } else {
                        None
                    }
                })
                .collect()
        })
        .collect()
}
